@@ -23,7 +23,7 @@ use AutoLoader;
 use FileHandle;
 use vars qw($canonical $forgive_me $VERSION);
 
-$VERSION = '2.18';
+$VERSION = '2.23';
 *AUTOLOAD = \&AutoLoader::AUTOLOAD;		# Grrr...
 
 #
@@ -262,11 +262,18 @@ sub _store {
 	my $ret;
 	# Call C routine nstore or pstore, depending on network order
 	eval { $ret = &$xsptr(*FILE, $self) };
-	close(FILE) or $ret = undef;
-	unlink($file) or warn "Can't unlink $file: $!\n" if $@ || !defined $ret;
+	# close will return true on success, so the or short-circuits, the ()
+	# expression is true, and for that case the block will only be entered
+	# if $@ is true (ie eval failed)
+	# if close fails, it returns false, $ret is altered, *that* is (also)
+	# false, so the () expression is false, !() is true, and the block is
+	# entered.
+	if (!(close(FILE) or undef $ret) || $@) {
+		unlink($file) or warn "Can't unlink $file: $!\n";
+	}
 	logcroak $@ if $@ =~ s/\.?\n$/,/;
 	$@ = $da;
-	return $ret ? $ret : undef;
+	return $ret;
 }
 
 #
@@ -305,7 +312,7 @@ sub _store_fd {
 	logcroak $@ if $@ =~ s/\.?\n$/,/;
 	local $\; print $file '';	# Autoflush the file if wanted
 	$@ = $da;
-	return $ret ? $ret : undef;
+	return $ret;
 }
 
 #
@@ -1045,7 +1052,7 @@ your data.  There is no slowdown on retrieval.
 
 =head1 BUGS
 
-You can't store GLOB, FORMLINE, etc.... If you can define semantics
+You can't store GLOB, FORMLINE, REGEXP, etc.... If you can define semantics
 for those operations, feel free to enhance Storable so that it can
 deal with them.
 
@@ -1177,7 +1184,7 @@ Storable was written by Raphael Manfredi F<E<lt>Raphael_Manfredi@pobox.comE<gt>>
 Maintenance is now done by the perl5-porters F<E<lt>perl5-porters@perl.orgE<gt>>
 
 Please e-mail us with problems, bug fixes, comments and complaints,
-although if you have complements you should send them to Raphael.
+although if you have compliments you should send them to Raphael.
 Please don't e-mail Raphael with problems, as he no longer works on
 Storable, and your message will be delayed while he forwards it to us.
 
